@@ -11,17 +11,28 @@ import gtk, gtk.glade
 import gnome.ui
 import os.path
 import gettext
+import threading
 from gettext import gettext as _
 import logging
 log = logging.getLogger("gtkui.UI")
 
 import conduit
-from conduit.gtkui.Canvas import Canvas
-from conduit.gtkui.Tree import DataProviderTreeModel, DataProviderTreeView
-from conduit.gtkui.ConflictResolver import ConflictResolver
-from conduit.gtkui.Database import GenericDBListStore
+import conduit.Web as Web
+import conduit.Conduit as Conduit
+import conduit.gtkui.Canvas as Canvas
+import conduit.gtkui.Tree as Tree
+import conduit.gtkui.ConflictResolver as ConflictResolver
+import conduit.gtkui.Database as Database
+
 
 DEFAULT_CONDUIT_BROWSER = "gtkmozembed"
+DEVELOPER_WEB_LINKS = (
+#name,                      #url
+("Introduction",            "http://www.conduit-project.org/wiki/Development"),
+("Writing a Data Provider", "http://www.conduit-project.org/wiki/WritingADataProvider"),
+("API Documentation",       "http://doc.conduit-project.org/conduit/"),
+("Test Results",            "http://tests.conduit-project.org/")
+)
 
 #set up the gettext system and locales
 for module in gtk.glade, gettext:
@@ -37,7 +48,6 @@ class MainWindow:
         Constructs the mainwindow. Throws up a splash screen to cover 
         the most time consuming pieces
         """
-        gnome.init(conduit.APPNAME, conduit.APPVERSION, properties={gnome.PARAM_APP_DATADIR:'/usr/share'})
         gnome.ui.authentication_manager_init()        
 
         self.conduitApplication = conduitApplication
@@ -52,14 +62,14 @@ class MainWindow:
                     ]
         for i in icon_dirs:                    
             gtk.icon_theme_get_default().prepend_search_path(i)
-            log.debug("Adding %s to icon theme search path" % (i))
 
         self.gladeFile = os.path.join(conduit.SHARED_DATA_DIR, "conduit.glade")
         self.widgets = gtk.glade.XML(self.gladeFile, "MainWindow")
         
         dic = { "on_mainwindow_delete" : self.on_window_closed,
                 "on_mainwindow_state_event" : self.on_window_state_event,
-                "on_synchronize_activate" : self.on_synchronize_all_clicked,      
+                "on_synchronize_activate" : self.on_synchronize_all_clicked,
+                "on_cancel_activate" : self.on_cancel_all_clicked,  
                 "on_quit_activate" : self.on_window_closed,
                 "on_clear_canvas_activate" : self.on_clear_canvas,
                 "on_preferences_activate" : self.on_conduit_preferences,
@@ -68,34 +78,35 @@ class MainWindow:
                 "on_save1_activate" : self.save_settings,
                 None : None
                 }
-         
         self.widgets.signal_autoconnect(dic)
 
         #type converter and sync manager
         self.type_converter = typeConverter
         self.sync_manager = syncManager
-        self.sync_manager.set_twoway_policy({
-                "conflict"  :   conduit.GLOBALS.settings.get("twoway_policy_conflict"),
-                "deleted"   :   conduit.GLOBALS.settings.get("twoway_policy_deleted")}
-                )
         
         #Initialize the mainWindow
         self.mainWindow = self.widgets.get_widget("MainWindow")
-        title = "%s" % conduit.APPNAME
+        #Enable RGBA colormap
+        if conduit.GLOBALS.settings.get("gui_use_rgba_colormap") == True:
+            screen = self.mainWindow.get_screen()
+            colormap = screen.get_rgba_colormap()
+            if colormap:
+                gtk.widget_set_default_colormap(colormap)
+        self.mainWindow.set_position(gtk.WIN_POS_CENTER)
+        self.mainWindow.set_icon_name("conduit")
+        title = "Conduit"
         if conduit.IS_DEVELOPMENT_VERSION:
-            title = title + " - %s (Development Version)" % conduit.APPVERSION
+            title = title + " - %s (Development Version)" % conduit.VERSION
         if not conduit.IS_INSTALLED:
             title = title + " - Running Uninstalled"
         self.mainWindow.set_title(title)
-        self.mainWindow.set_position(gtk.WIN_POS_CENTER)
-        self.mainWindow.set_icon_name("conduit")
-        
+
         #Configure canvas
         self.canvasSW = self.widgets.get_widget("canvasScrolledWindow")
         self.hpane = self.widgets.get_widget("hpaned1")
 
         #start up the canvas
-        self.canvas = Canvas(
+        self.canvas = Canvas.Canvas(
                         parentWindow=self.mainWindow,
                         typeConverter=self.type_converter,
                         syncManager=self.sync_manager,
@@ -107,39 +118,57 @@ class MainWindow:
         self.canvas.connect("drag-data-received", self.drag_data_received_data)
         
         # Populate the tree model
-        self.dataproviderTreeModel = DataProviderTreeModel() 
+        self.dataproviderTreeModel = Tree.DataProviderTreeModel() 
         dataproviderScrolledWindow = self.widgets.get_widget("scrolledwindow2")
-        self.dataproviderTreeView = DataProviderTreeView(self.dataproviderTreeModel)
+        self.dataproviderTreeView = Tree.DataProviderTreeView(self.dataproviderTreeModel)
         dataproviderScrolledWindow.add(self.dataproviderTreeView)
 
         #Set up the expander used for resolving sync conflicts
-        self.conflictResolver = ConflictResolver(self.widgets)
-
-        #setup the module manager
-        self.moduleManager = moduleManager
-        self.dataproviderTreeModel.add_dataproviders(self.moduleManager.get_modules_by_type("source"))
-        self.dataproviderTreeModel.add_dataproviders(self.moduleManager.get_modules_by_type("sink"))
-        self.dataproviderTreeModel.add_dataproviders(self.moduleManager.get_modules_by_type("twoway"))
-        self.moduleManager.connect("dataprovider-available", self.on_dataprovider_available)
-        self.moduleManager.connect("dataprovider-unavailable", self.on_dataprovider_unavailable)
+        self.conflictResolver = ConflictResolver.ConflictResolver(self.widgets)
 
         #final GUI setup
+        self.cancelSyncButton = self.widgets.get_widget('cancel')
         self.hpane.set_position(conduit.GLOBALS.settings.get("gui_hpane_postion"))
-        self.dataproviderTreeView.expand_all()
+        self.dataproviderTreeView.set_expand_rows()
         self.window_state = 0
-
+        
+        #if running a development version, add some developer specific links
+        #to the help menu
+        if conduit.IS_DEVELOPMENT_VERSION:
+            helpMenu = self.widgets.get_widget("help_menu")
+            developersMenuItem = gtk.ImageMenuItem("Developers")
+            developersMenuItem.set_image(
+                                gtk.image_new_from_icon_name(
+                                        "applications-development",
+                                        gtk.ICON_SIZE_MENU))
+            developersMenu = gtk.Menu()
+            developersMenuItem.set_submenu(developersMenu)
+            helpMenu.prepend(developersMenuItem)
+            for name,url in DEVELOPER_WEB_LINKS:
+                item = gtk.ImageMenuItem(name)
+                item.set_image(
+                        gtk.image_new_from_icon_name(
+                                "applications-internet",
+                                gtk.ICON_SIZE_MENU))
+                item.connect("activate",self.on_developer_menu_item_clicked,name,url)
+                developersMenu.append(item)
+                
+                
+    def on_developer_menu_item_clicked(self, menuitem, name, url):
+        threading.Thread(
+                    target=Web.LoginMagic,
+                    args=(name, url),
+                    kwargs={"login_function":lambda: True}
+                    ).start()
+        
     def on_conduit_added(self, syncset, cond):
         cond.connect("sync-started", self.on_sync_started)
         cond.connect("sync-completed", self.on_sync_completed)
         cond.connect("sync-conflict", self.conflictResolver.on_conflict)
 
-    def on_conduit_removed(self, syncset, cond):
-        pass
-
     def set_model(self, syncSet):
         self.syncSet = syncSet
         self.syncSet.connect("conduit-added", self.on_conduit_added)
-        self.syncSet.connect("conduit-removed", self.on_conduit_removed)
         self.canvas.set_sync_set(syncSet)
 
     def present(self):
@@ -166,33 +195,25 @@ class MainWindow:
         return (not minimized) and self.mainWindow.get_property('visible')
 
     def on_sync_started(self, thread):
-        log.debug("GUI got sync started")
+        self.cancelSyncButton.set_property("sensitive", True)
 
     def on_sync_completed(self, thread, aborted, error, conflict):
-        log.debug("GUI got sync completed")
+        self.cancelSyncButton.set_property(
+                "sensitive",
+                conduit.GLOBALS.syncManager.is_busy()
+                )
        
-    def on_dataprovider_available(self, loader, dataprovider):
-        """
-        Adds the new dataprovider to the treeview
-        """
-        if dataprovider.enabled == True:
-            self.dataproviderTreeModel.add_dataprovider(dataprovider)
-
-    def on_dataprovider_unavailable(self, unloader, dataprovider):
-        """
-        Removes the dataprovider from the treeview and replaces it with pending dataproviders
-        """
-        self.dataproviderTreeModel.remove_dataprovider(dataprovider)
-        
     def on_synchronize_all_clicked(self, widget):
         """
         Synchronize all valid conduits on the canvas
         """
-        for conduit in self.syncSet.get_all_conduits():
-            if conduit.datasource is not None and len(conduit.datasinks) > 0:
-                self.sync_manager.sync_conduit(conduit)
-            else:
-                log.info("Conduit must have a datasource and a datasink")        
+        self.conduitApplication.Synchronize()
+                
+    def on_cancel_all_clicked(self, widget):
+        """
+        Cancels all currently runnings syncs
+        """
+        self.conduitApplication.Cancel()
 
     def on_clear_canvas(self, widget):
         """
@@ -205,25 +226,6 @@ class MainWindow:
         Show the properties of the current sync set (status, conflicts, etc
         Edit the sync specific properties
         """
-        def restore_policy_state(policy, ask_rb, replace_rb, skip_rb):
-            pref = conduit.GLOBALS.settings.get("twoway_policy_%s" % policy)
-            if pref == "skip":
-                skip_rb.set_active(True)
-            elif pref == "replace":
-                replace_rb.set_active(True)
-            else:
-                ask_rb.set_active(True)
-
-        def save_policy_state(policy, ask_rb, replace_rb, skip_rb):
-            if skip_rb.get_active() == True:
-                i = "skip"
-            elif replace_rb.get_active() == True:
-                i = "replace"
-            else:            
-                i = "ask"
-            conduit.GLOBALS.settings.set("twoway_policy_%s" % policy, i)
-            return i
-
         def on_clear_button_clicked(sender, treeview, sqliteListStore):
             treeview.set_model(None)
             conduit.GLOBALS.mappingDB.delete()
@@ -240,10 +242,10 @@ class MainWindow:
             converterListStore.append( (string,) )
         dataProviderListStore = gtk.ListStore( str, bool )
         #get all dataproviders
-        for i in self.moduleManager.get_modules_by_type("sink","source","twoway"):
+        for i in conduit.GLOBALS.moduleManager.get_modules_by_type("sink","source","twoway"):
             dataProviderListStore.append(("Name: %s\nDescription: %s)" % (i.name, i.description), True))
         #include files that could not be loaded
-        for f in self.moduleManager.invalidFiles:
+        for f in conduit.GLOBALS.moduleManager.invalidFiles:
             dataProviderListStore.append(("Error loading file: %s" % f, False))
 
         #construct the dialog
@@ -270,7 +272,7 @@ class MainWindow:
                 treeview.append_column(column)
                 index = index + 1
 
-            store = GenericDBListStore("mappings", db)
+            store = Database.GenericDBListStore("mappings", db)
             treeview.set_model(store)            
             
             sw = gtk.ScrolledWindow()
@@ -311,17 +313,18 @@ class MainWindow:
         web_browser_check = tree.get_widget("web_check")
         web_browser_check.set_active(conduit.GLOBALS.settings.get("web_login_browser") != "system")
 
-
-        #get the radiobuttons where the user sets their policy
-        conflict_ask_rb = tree.get_widget("conflict_ask_rb")
-        conflict_replace_rb = tree.get_widget("conflict_replace_rb")
-        conflict_skip_rb = tree.get_widget("conflict_skip_rb")
-        deleted_ask_rb = tree.get_widget("deleted_ask_rb")
-        deleted_replace_rb = tree.get_widget("deleted_replace_rb")
-        deleted_skip_rb = tree.get_widget("deleted_skip_rb")
-        #set initial conditions        
-        restore_policy_state("conflict", conflict_ask_rb, conflict_replace_rb, conflict_skip_rb)
-        restore_policy_state("deleted", deleted_ask_rb, deleted_replace_rb, deleted_skip_rb)
+        #restore the current policy
+        for policyName in Conduit.CONFLICT_POLICY_NAMES:
+            currentValue = conduit.GLOBALS.settings.get("default_policy_%s" % policyName)
+            for policyValue in Conduit.CONFLICT_POLICY_VALUES:
+                widgetName = "%s_%s" % (policyName,policyValue)
+                widget = tree.get_widget(widgetName)
+                widget.set_image(
+                        gtk.image_new_from_icon_name(
+                                Conduit.CONFLICT_POLICY_VALUE_ICONS[widgetName],
+                                gtk.ICON_SIZE_MENU))
+                if currentValue == policyValue:
+                    widget.set_active(True)
                                         
         #Show the dialog
         dialog = tree.get_widget("PreferencesDialog")
@@ -335,10 +338,14 @@ class MainWindow:
                 conduit.GLOBALS.settings.set("web_login_browser", DEFAULT_CONDUIT_BROWSER)
             else:
                 conduit.GLOBALS.settings.set("web_login_browser", "system")
-            policy = {}
-            policy["conflict"] = save_policy_state("conflict", conflict_ask_rb, conflict_replace_rb, conflict_skip_rb)
-            policy["deleted"] = save_policy_state("deleted", deleted_ask_rb, deleted_replace_rb, deleted_skip_rb)
-            self.sync_manager.set_twoway_policy(policy)
+            #save the current policy
+            for policyName in Conduit.CONFLICT_POLICY_NAMES:
+                for policyValue in Conduit.CONFLICT_POLICY_VALUES:
+                    widgetName = "%s_%s" % (policyName,policyValue)
+                    if tree.get_widget(widgetName).get_active() == True:
+                        conduit.GLOBALS.settings.set(
+                                "default_policy_%s" % policyName,
+                                policyValue)
         dialog.destroy()                
 
 
@@ -355,7 +362,16 @@ class MainWindow:
         """
         Display help
         """
-        gnome.help_display(conduit.APPNAME, None)
+        if conduit.IS_INSTALLED:
+            uri = "ghelp:conduit"
+        else:
+            #if we are not installed then launch the ghelp uri with a full path
+            uri = "ghelp:%s" % os.path.join(conduit.DIRECTORY,"help","C","conduit.xml")
+        log.debug("Launching help: %s" % uri)
+        gobject.spawn_async(
+                    argv=("xdg-open",uri),
+                    flags=gobject.SPAWN_SEARCH_PATH | gobject.SPAWN_STDOUT_TO_DEV_NULL | gobject.SPAWN_STDERR_TO_DEV_NULL
+                    )
 
     def on_window_state_event(self, widget, event):
         visible = self.is_visible()
@@ -396,11 +412,6 @@ class MainWindow:
         #DBus interface which will tidy up any pending running
         #non gui tasks
         if quit:
-            #FIXME: I want to do this call over DBus but this hangs. Why?
-            #sessionBus = dbus.SessionBus()
-            #obj = sessionBus.get_object(conduit.DBUS_IFACE, "/activate")
-            #conduitApp = dbus.Interface(obj, conduit.DBUS_IFACE)
-            #conduitApp.Quit()
             self.conduitApplication.Quit()
         
         
@@ -420,7 +431,7 @@ class MainWindow:
         #signal and NOT here
         if dataproviderKey != "":
             #Add a new instance if the dataprovider to the canvas.
-            new = self.moduleManager.get_module_wrapper_with_instance(dataproviderKey)
+            new = conduit.GLOBALS.moduleManager.get_module_wrapper_with_instance(dataproviderKey)
             self.canvas.add_dataprovider_to_canvas(dataproviderKey, new, x, y)
         
         context.finish(True, True, etime)
@@ -435,8 +446,15 @@ class MainWindow:
         self.syncSet.save_to_xml()
 
         #GUI settings
-        conduit.GLOBALS.settings.set("gui_hpane_postion", self.hpane.get_position())
-        conduit.GLOBALS.settings.set("gui_window_size", self.mainWindow.get_size())
+        conduit.GLOBALS.settings.set(
+                            "gui_hpane_postion",
+                            self.hpane.get_position())
+        conduit.GLOBALS.settings.set(
+                            "gui_window_size",
+                            self.mainWindow.get_size())
+        conduit.GLOBALS.settings.set(
+                            "gui_expanded_rows",
+                            self.dataproviderTreeView.get_expanded_rows())        
 
 class SplashScreen:
     """
@@ -497,11 +515,19 @@ class SplashScreen:
 class AboutDialog(gtk.AboutDialog):
     def __init__(self):
         gtk.AboutDialog.__init__(self)
-        self.set_name(conduit.APPNAME)
-        self.set_version(conduit.APPVERSION)
+        self.set_name("Conduit")
+        self.set_version(conduit.VERSION)
         self.set_comments("Synchronisation for GNOME")
         self.set_website("http://www.conduit-project.org")
-        self.set_authors(["John Stowers","John Carr","Thomas Van Machelen","Jonny Lamb"])
+        self.set_authors([
+        		"John Stowers",
+        		"John Carr",
+        		"Thomas Van Machelen",
+        		"Jonny Lamb"])
+        self.set_artists([
+        		"John Stowers",
+        		"mejogid",
+        		"The Tango Project (http://tango.freedesktop.org)"])
         self.set_logo_icon_name("conduit")
 
 class StatusIcon(gtk.StatusIcon):
@@ -517,6 +543,7 @@ class StatusIcon(gtk.StatusIcon):
              <menubar name="Menubar">
               <menu action="Menu">
                <menuitem action="Sync"/>
+               <menuitem action="Cancel"/>
                <menuitem action="Quit"/>
                <separator/>
                <menuitem action="About"/>
@@ -526,7 +553,8 @@ class StatusIcon(gtk.StatusIcon):
         '''
         actions = [
             ('Menu',  None, 'Menu'),
-            ('Sync', gtk.STOCK_EXECUTE, _("_Synchronize"), None, _("Synchronize all dataproviders"), self.on_synchronize),
+            ('Sync', gtk.STOCK_EXECUTE, _("_Synchronize All"), None, _("Synchronizes All Groups"), self.on_synchronize),
+            ('Cancel', gtk.STOCK_CANCEL, _("_Cancel Synchronization"), None, _("Cancels Currently Synchronizing Groups"), self.on_cancel),
             ('Quit', gtk.STOCK_QUIT, _("_Quit"), None, _("Close Conduit"), self.on_quit),
             ('About', gtk.STOCK_ABOUT, _("_About"), None, _("About Conduit"), self.on_about)]
         ag = gtk.ActionGroup('Actions')
@@ -534,86 +562,80 @@ class StatusIcon(gtk.StatusIcon):
         self.manager = gtk.UIManager()
         self.manager.insert_action_group(ag, 0)
         self.manager.add_ui_from_string(menu)
-        self.menu = self.manager.get_widget('/Menubar/Menu/About').props.parent        
+        self.menu = self.manager.get_widget('/Menubar/Menu/About').props.parent
+        self.cancelButton = self.manager.get_widget('/Menubar/Menu/Cancel')   
         self.connect('popup-menu', self.on_popup_menu)
         self.connect('activate', self.on_click)
 
-        #start with the application icon
-        self._reset_states()
-        self._go_to_idle_state()
-        self.set_tooltip("Conduit")
-        self.set_visible(True)
-
-    def _reset_states(self):
-        self.running = 0
+        self.animating = False
+        self.check_animate = False
         self.conflict = False
         self.animated_idx = 0
         self.animated_icons = range(1,8)
 
-    def _go_to_idle_state(self):
+        #start with the application icon
+        self.cancelButton.set_property("sensitive", False)
         self.set_from_icon_name("conduit")
-        self.set_tooltip(_("Synchronization Complete"))
+        self.set_tooltip("Conduit")
+        self.set_visible(True)
 
-    def _go_to_conflict_state(self):
-        self.set_from_icon_name("dialog-error")
-        self.set_tooltip(_("Synchronization Error"))
-
-    def _go_to_running_state(self):
-        self.set_tooltip(_("Synchronizing"))
-        if self.animated_idx == self.animated_icons[-1]:
-            self.animated_idx = 1
-        else:
-            self.animated_idx += 1
-        self.set_from_icon_name("conduit-progress-%d" % self.animated_idx)
-        if self.running == 0:
-            if self.conflict:
-                self._go_to_conflict_state()
-                self._reset_states()
+    def _animate_icon_timeout(self):
+        #FIXME: When will gtk support animated gtk status icons?
+        if self.animating:
+            if self.animated_idx == self.animated_icons[-1]:
+                self.animated_idx = 1
             else:
-                self._go_to_idle_state()
-                self._reset_states()
-
-            return False
-        else:
+                self.animated_idx += 1
+                self.set_from_icon_name("conduit-progress-%d" % self.animated_idx)
+            # re-check animation?                
+            if self.check_animate:
+                self.animating =  conduit.GLOBALS.syncManager.is_busy()
+                self.check_animate = False
             return True
 
+        else: 
+            if self.conflict:
+                self.set_from_icon_name("dialog-error")
+                self.set_tooltip(_("Synchronization Error"))
+            else:
+                self.set_from_icon_name("conduit")
+                self.set_tooltip(_("Synchronization Complete"))
+            self.conflict = False
+            self.cancelButton.set_property("sensitive", False)
+            return False
+
     def on_conduit_added(self, syncset, cond):
         cond.connect("sync-started", self._on_sync_started)
         cond.connect("sync-completed", self._on_sync_completed)
         cond.connect("sync-conflict", self._on_sync_conflict)
-        #cond.connect("sync-progress", self._on_sync_progress)
-
+        
     def on_conduit_removed(self, syncset, cond):
         pass
 
     def _on_sync_started(self, cond):
-        self.running += 1
-        gobject.timeout_add(100, self._go_to_running_state)
-        log.debug("Icon got sync started")
+        if not self.animating:
+            self.animating = True
+            self.set_tooltip(_("Synchronizing"))
+            self.cancelButton.set_property("sensitive", True)
+            gobject.timeout_add(100, self._animate_icon_timeout)
 
     def _on_sync_completed(self, cond, aborted, error, conflict):
-        self.running -= 1
-        log.debug("Icon got sync completed %s (error: %s)" % (self.running, error))
+        # check need for animation on next iteration
+        self.check_animate = True
 
     def _on_sync_conflict(self, cond, conflict):
         self.conflict = True
-        log.debug("Icon got sync conflict")
 
     def on_synchronize(self, data):
-        #sessionBus = dbus.SessionBus()
-        #obj = sessionBus.get_object(conduit.DBUS_IFACE, "/activate")
-        #conduitApp = dbus.Interface(obj, conduit.DBUS_IFACE)
-        #conduitApp.Synchronize()
         self.conduitApplication.Synchronize()
+        
+    def on_cancel(self, data):
+        self.conduitApplication.Cancel()
 
     def on_popup_menu(self, status, button, time):
         self.menu.popup(None, None, None, button, time)
 
     def on_quit(self, data):
-        #sessionBus = dbus.SessionBus()
-        #obj = sessionBus.get_object(conduit.DBUS_IFACE, "/activate")
-        #conduitApp = dbus.Interface(obj, conduit.DBUS_IFACE)
-        #conduitApp.Quit()
         self.conduitApplication.Quit()
 
     def on_about(self, data):
