@@ -1,9 +1,10 @@
+import os
 import re
+import urlparse
 import gobject
 import datetime
 import dateutil.parser
 import vobject
-import operator
 import time
 from dateutil.tz import tzutc, tzlocal
 from gettext import gettext as _
@@ -26,19 +27,19 @@ import conduit.datatypes.File as File
 #and the appropriate directories
 Utils.dataprovider_add_dir_to_path(__file__)
 try:
-    import atom
-    import gdata
+    import atom.service
     import gdata.service
-    import gdata.photos
     import gdata.photos.service    
-    import gdata.calendar
     import gdata.calendar.service
+    import gdata.contacts.service
+    import gdata.docs.service
 
     MODULES = {
         "GoogleCalendarTwoWay" : { "type": "dataprovider" },
         "PicasaTwoWay" :         { "type": "dataprovider" },
         "YouTubeSource" :        { "type": "dataprovider" },    
-        "ContactsSource" :       { "type": "dataprovider" },    
+        "ContactsTwoWay" :       { "type": "dataprovider" },
+#        "DocumentsSink" :        { "type": "dataprovider" },
     }
     log.info("Module Information: %s" % Utils.get_module_information(gdata, None))
 except (ImportError, AttributeError):
@@ -48,22 +49,35 @@ except (ImportError, AttributeError):
 # time format
 FORMAT_STRING = "%Y-%m-%dT%H:%M:%S"
 
-class GoogleBase(object):
-    def __init__(self):
+class _GoogleBase:
+    def __init__(self, service):
         self.username = ""
         self.password = ""
         self.loggedIn = False
+        self.service = service
         
+        if conduit.GLOBALS.settings.proxy_enabled():
+            log.info("Configuring proxy for %s" % self.service)
+            host,port,user,password = conduit.GLOBALS.settings.get_proxy()
+            #FIXME: Is this necessary, does GNOME propogate the gconf vars to 
+            #env vars? gdata automatically picks those up
+            os.environ['http_proxy'] = "%s:%s" % (host,port)
+            os.environ['https_proxy'] = "%s:%s" % (host,port)
+            os.environ['proxy_username'] = user
+            os.environ['proxy_password'] = password
+
     def _do_login(self):
-        raise NotImplementedError
+        self.service.ClientLogin(self.username, self.password)
 
     def _login(self):
         if not self.loggedIn:
             try:
                 self._do_login()
                 self.loggedIn = True
-            except:
-                self.loggedIn = False
+            except gdata.service.BadAuthentication:
+                log.info("Error logging in: Incorrect username or password")
+            except Exception, e:
+                log.info("Error logging in: %s" % e)
        
     def _set_username(self, username):
         if self.username != username:
@@ -95,7 +109,7 @@ class GoogleBase(object):
     def get_UID(self):
         return self.username
 
-class GoogleCalendar(object):
+class _GoogleCalendar:
     def __init__(self, name, uri):
         self.uri = uri
         self.name = name
@@ -122,8 +136,7 @@ class GoogleCalendar(object):
         return '/calendar/feeds/' + self.get_uri() + '/private/full'
 
 def convert_madness_to_datetime(inputDate):
-    log.debug('Attempting to parse the following: %s'%inputDate)
-    logging.debug('Attempting to parse the following: %s'%inputDate)
+    log.debug('Attempting to parse: %s' % inputDate)
     dateStr = None
     dateDate = None
     dateDateTime = None
@@ -149,7 +162,7 @@ def convert_madness_to_datetime(inputDate):
         
     if dateDateTime is not None:
         if dateDateTime.tzinfo is not None:
-            logging.warn("returning: %s",dateDateTime)
+            log.warn("returning: %s",dateDateTime)
             ts = dateDateTime.timetuple()
             dateDateTime = dateDateTime.fromtimestamp(time.mktime(ts))
             return dateDateTime
@@ -186,7 +199,7 @@ def parse_google_recur(recurString, args):
     if 'vtimezone' in vobjICal.contents:
         args['vtimezone'] = vobjICal.vtimezone
 
-class GoogleEvent(object):
+class _GoogleEvent:
     def __init__(self, **kwargs):
         self.uid = kwargs.get('uid', None)
         self.mTime = kwargs.get('mTime', None)
@@ -349,7 +362,7 @@ class GoogleEvent(object):
         log.debug("Created ICal Format :\n"+returnStr)
         return returnStr
     
-class GoogleCalendarTwoWay(GoogleBase, DataProvider.TwoWay):
+class GoogleCalendarTwoWay(_GoogleBase, DataProvider.TwoWay):
 
     _name_ = _("Google Calendar")
     _description_ = _("Sync your Google Calendar")
@@ -360,29 +373,23 @@ class GoogleCalendarTwoWay(GoogleBase, DataProvider.TwoWay):
     _icon_ = "appointment-new"
     
     def __init__(self):
-        GoogleBase.__init__(self)
+        _GoogleBase.__init__(self,gdata.calendar.service.CalendarService())
         DataProvider.TwoWay.__init__(self)
-        self.calService = gdata.calendar.service.CalendarService()
         self.selectedCalendar = None
         self.events = {}
 
-    def _do_login(self):
-        self.calService.email = self.username
-        self.calService.password = self.password
-        self.calService.ProgrammaticLogin()
-
     def _get_all_events(self):
         self._login()
         calQuery = gdata.calendar.service.CalendarEventQuery(user = self.selectedCalendar.get_uri())
-        eventFeed = self.calService.CalendarQuery(calQuery)
+        eventFeed = self.service.CalendarQuery(calQuery)
         for event in eventFeed.entry:   
-            yield GoogleEvent.from_google_format(event)
+            yield _GoogleEvent.from_google_format(event)
 
     def _get_all_calendars(self):
         self._login()
-        allCalendarsFeed = self.calService.GetCalendarListFeed().entry
+        allCalendarsFeed = self.service.GetCalendarListFeed().entry
         for calendarFeed in allCalendarsFeed:
-            yield GoogleCalendar.from_google_format(calendarFeed)
+            yield _GoogleCalendar.from_google_format(calendarFeed)
 
     def _load_calendars(self, widget, tree):
         import gtk, gtk.gdk
@@ -474,16 +481,16 @@ class GoogleCalendarTwoWay(GoogleBase, DataProvider.TwoWay):
         return conduitEvent          
                    
     def _create_event(self, conduitEvent):
-        googleEvent = GoogleEvent.from_ical_format( conduitEvent.get_ical_string() )
-        newEvent = self.calService.InsertEvent(
+        googleEvent = _GoogleEvent.from_ical_format( conduitEvent.get_ical_string() )
+        newEvent = self.service.InsertEvent(
                                         googleEvent.get_google_format(),
                                         self.selectedCalendar.get_feed_link())
-        newEvent = GoogleEvent.from_google_format(newEvent)
+        newEvent = _GoogleEvent.from_google_format(newEvent)
         return Rid(uid=newEvent.get_uid(), mtime=None, hash=None)
         
     def _delete_event(self, LUID):
         googleEvent = self.events[LUID]
-        self.calService.DeleteEvent(googleEvent.get_edit_link())
+        self.service.DeleteEvent(googleEvent.get_edit_link())
         
     def _update_event(self, LUID, conduitEvent):
         self._delete_event(LUID)
@@ -518,7 +525,7 @@ class GoogleCalendarTwoWay(GoogleBase, DataProvider.TwoWay):
         return rid
         
     def get_configuration(self):
-        conf = GoogleBase.get_configuration(self)
+        conf = _GoogleBase.get_configuration(self)
         if self.selectedCalendar != None:
             conf.update({
                 "selectedCalendarName"  :   self.selectedCalendar.get_name(),
@@ -526,33 +533,32 @@ class GoogleCalendarTwoWay(GoogleBase, DataProvider.TwoWay):
         return conf
             
     def set_configuration(self, config):
-        GoogleBase.set_configuration(self, config)
+        _GoogleBase.set_configuration(self, config)
         if "selectedCalendarName" in config:
             if "selectedCalendarURI" in config:
-                self.selectedCalendar = GoogleCalendar(
+                self.selectedCalendar = _GoogleCalendar(
                                             config['selectedCalendarName'],
                                             config['selectedCalendarURI']
                                             )
 
     def is_configured (self, isSource, isTwoWay):
-        if not GoogleBase.is_configured(self, isSource, isTwoWay):
+        if not _GoogleBase.is_configured(self, isSource, isTwoWay):
             return False
         if self.selectedCalendar == None:
             return False
         return True
 
-class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
+class PicasaTwoWay(_GoogleBase, Image.ImageTwoWay):
 
     _name_ = _("Picasa")
     _description_ = _("Sync your Google Picasa photos")
     _icon_ = "picasa"
 
     def __init__(self, *args):
-        GoogleBase.__init__(self)
+        _GoogleBase.__init__(self, gdata.photos.service.PhotosService())
         Image.ImageTwoWay.__init__(self)
         self.albumName = ""
         self.imageSize = "None"
-        self.pws = gdata.photos.service.PhotosService()
         self.galbum = None
         self.gphoto_dict = {}
 
@@ -570,20 +576,17 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
         
     def _upload_photo (self, uploadInfo):
         try:
-            gphoto = self.pws.InsertPhotoSimple(
+            gphoto = self.service.InsertPhotoSimple(
                                 self.galbum,
                                 uploadInfo.name,
-                                '',
+                                uploadInfo.caption,
                                 uploadInfo.url)
             for tag in uploadInfo.tags:
-                self.pws.InsertTag(gphoto, str(tag))
+                self.service.InsertTag(gphoto, str(tag))
             return Rid(uid=gphoto.gphoto_id.text)
         except Exception, e:
             raise Exceptions.SyncronizeError("Picasa Upload Error.")
 
-    def _do_login(self):
-        self.pws.ClientLogin(self.username, self.password)
-
     def _get_album(self):
         for name,album in self._get_albums():
             if name == self.albumName:
@@ -596,7 +599,7 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
 
     def _get_albums(self):
         albums = []
-        for album in self.pws.GetUserFeed().entry:
+        for album in self.service.GetUserFeed().entry:
             albums.append(
                     (album.title.text,  #album name
                     album))             #album
@@ -604,7 +607,7 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
         
     def _get_photos(self):
         self.gphoto_dict = {}
-        for photo in self.pws.GetFeed(self.galbum.GetPhotosUri()).entry:
+        for photo in self.service.GetFeed(self.galbum.GetPhotosUri()).entry:
             self.gphoto_dict[photo.gphoto_id.text] = photo
 
     def _get_photo_timestamp(self, gphoto):
@@ -617,14 +620,17 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
             return datetime(*(time.strptime(timestamp, FORMAT_STRING)[0:6]))
 
     def _create_album(self, album_name):            
-        self.pws.InsertAlbum(album_name, '', access='private')
+        self.service.InsertAlbum(album_name, '', access='private')
 
     def refresh(self):
         Image.ImageTwoWay.refresh(self)
         self._login()
+        if not self.loggedIn:
+            raise Exceptions.RefreshError("Could not log in")
         self._get_album()
 
     def get_all (self):
+        Image.ImageTwoWay.get_all(self)
         self._get_photos()
         return self.gphoto_dict.keys()
         
@@ -633,13 +639,14 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
 
         gphoto = self.gphoto_dict[LUID]
         url = gphoto.GetMediaURL()
-        tags = (tag.title.text for tag in self.pws.GetFeed(gphoto.GetTagsUri()).entry)
+        tags = (tag.title.text for tag in self.service.GetFeed(gphoto.GetTagsUri()).entry)
 
         f = Photo.Photo (URI=url)
         f.force_new_mtime(self._get_photo_timestamp(gphoto))
         f.set_open_URI(url)
         f.set_UID(LUID)
         f.set_tags(tags)
+        f.set_caption(gphoto.summary.text)
         return f
 
     def delete(self, LUID):
@@ -647,7 +654,7 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
             log.warn("Photo does not exit")
             return
 
-        self.pws.Delete(self.gphoto_dict[LUID])
+        self.service.Delete(self.gphoto_dict[LUID])
         del self.gphoto_dict[LUID]
 
     def configure(self, window):
@@ -738,111 +745,224 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
         Utils.run_dialog_non_blocking(dlg, on_response, window)
         
     def get_configuration(self):
-        conf = GoogleBase.get_configuration(self)
+        conf = _GoogleBase.get_configuration(self)
         conf.update({
             "imageSize" :   self.imageSize,
             "album"     :   self.albumName})
         return conf
         
     def set_configuration(self, config):
-        GoogleBase.set_configuration(self, config)
+        _GoogleBase.set_configuration(self, config)
         self.imageSize = config.get("imageSize","None")
         self.albumName = config.get("album","")
 
     def is_configured (self, isSource, isTwoWay):
-        if not GoogleBase.is_configured(self, isSource, isTwoWay):
+        if not _GoogleBase.is_configured(self, isSource, isTwoWay):
             return False
         if len(self.albumName) < 1:
             return False
         return True
 
 
-class ContactsSource(GoogleBase, DataProvider.DataSource):
+class ContactsTwoWay(_GoogleBase,  DataProvider.TwoWay):
     """
     Contacts GData provider
     """
     _name_ = _("Google Contacts")
-    _description_ = _("Sync contacts from Google")
+    _description_ = _("Sync your Gmail contacts")
     _category_ = conduit.dataproviders.CATEGORY_OFFICE
-    _module_type_ = "source"
+    _module_type_ = "twoway"
     _out_type_ = "contact"
     _icon_ = "contact-new"
 
-    FEED = "http://www.google.com/m8/feeds/contacts/%s/base?max-results=25000"
-
     def __init__(self, *args):
-        GoogleBase.__init__(self)
-        DataProvider.DataSource.__init__(self)
-        self.entries = None
-        self.feed = ""
-
-    def _set_username(self, username):
-        GoogleBase._set_username(self, username)
-        self.feed = self.FEED % self.username
-
-    def _do_login(self):
-        self.service = gdata.service.GDataService(service="cp", server="www.google.com")
-        self.service.ClientLogin(self.username, self.password)
+        _GoogleBase.__init__(self,gdata.contacts.service.ContactsService())
+        DataProvider.TwoWay.__init__(self)
         
-    def _get_contact(self, LUID):
+    def _google_contact_from_conduit_contact(self, contact, gc=None):
+        """
+        Fills the apropriate fields in the google gdata contact type based on
+        those in the conduit contact type
+        """
+        name = contact.get_name()
+        emails = contact.get_emails()
+        #Google contacts must feature at least a name and an email address
+        if not (name and emails):
+            return None
+
+        #can also edit existing contacts
+        if not gc:
+            gc = gdata.contacts.ContactEntry()        
+        gc.title = atom.Title(text=name)
+
+        #Create all emails, make first one primary, if the contact doesnt
+        #already have a primary email address
+        primary = 'false'
+        existing = []
+        for ex in gc.email:
+            if ex.primary and ex.primary == 'true':
+                primary = 'true'
+                existing.append(ex)      
+        
+        for email in emails:
+            if email not in existing:
+                log.debug("Adding new email address %s %s" % (email, existing))
+                gc.email.append(gdata.contacts.Email(
+                                            address=email, 
+                                            primary=primary))#,rel=gdata.contacts.REL_WORK))
+                primary = 'false'
+        #notes = contact.get_notes()
+        #if notes: gc.content = atom.Content(text=notes)
+        
+        return gc
+
+        
+    def _conduit_contact_from_google_contact(self, gc):
+        """
+        Extracts available and interesting fields from the google contact
+        and stored them in the conduit contact type
+        """
+        c = Contact.Contact(formattedName=str(gc.title.text))
+        
+        emails = [str(e.address) for e in gc.email]
+        c.set_emails(*emails)
+        
+        #ee_names = map(operator.attrgetter('tag'),gc.extension_elements)
+        #if len(gc.extension_elements) >0:
+        #    for e in [e for e in ee_names if e == 'phoneNumber']:
+        #        c.vcard.add('tel')
+        #        c.vcard.tel.value = gc.extension_elements[ee_names.index('phoneNumber')].text
+        #        c.vcard.tel.type_param = gc.extension_elements[ee_names.index('phoneNumber')].attributes['rel'].split('#')[1]
+        #    for e in [e for e in ee_names if e == 'postalAddress']:
+        #        c.vcard.add('adr')
+        #        c.vcard.adr.value = vobject.vcard.Address(gc.extension_elements[ee_names.index('postalAddress')].text)
+        #        c.vcard.adr.type_param = gc.extension_elements[ee_names.index('postalAddress')].attributes['rel'].split('#')[1]
+        
+        return c
+
+    def _create_contact(self, contact):
+        gc = self._google_contact_from_conduit_contact(contact)
+        if not gc:
+            log.info("Could not create google contact from conduit contact")
+            return None
+
+        try:            
+            entry = self.service.CreateContact(gc)
+        except gdata.service.RequestError, e:
+            #If the response dict reson is 'Conflict' then we are trying to
+            #store a contact with the same email as one which already exists
+            if e.message.get("reason","") == "Conflict":
+                log.warn("FIXME: FIND THE OLD CONTACT BY EMAIL, GET IT, AND RAISE A CONFLICT EXCEPTION")
+                raise Exceptions.SynchronizeConflictError("FIXME", "FIXME", "FIXME")
+        except Exception, e:
+            log.warn("Error creating contact: %s" % e)
+            return None
+
+        if entry:
+            log.debug("Created contact: %s" % entry.id.text)
+            return entry.id.text
+        else:
+            log.debug("Create contact error")
+            return None
+
+    def _update_contact(self, LUID, contact):
         #get the gdata contact from google
-        gdc = self.service.Get(LUID)
-        if gdc is None or len(gdc.ToString()) < 1:
-            log.warn("Error getting/parsing gdata contact")
+        try:
+            oldgc = self.service.Get(LUID, converter=gdata.contacts.ContactEntryFromString)
+        except gdata.service.RequestError:
             return None
             
-        #FIXME: We should not be accessing the contact vcard directly.
-        #once we know what we can get from the gdata information, we should
-        #add the appropriate methods to the contact class to allow us to
-        #set these things
-        c = Contact.Contact()
+        #update the contact
+        gc = self._google_contact_from_conduit_contact(contact, oldgc)
+        self.service.UpdateContact(oldgc.GetEditLink().href, gc)
+        
+        #fixme, we should really just return the RID here, but its safer
+        #to use the same code path as get, because I am not sure if/how google
+        #changes the mtime
+        return LUID
+    
+    def _get_contact(self, LUID):
+        if not LUID:
+            return None
 
-        c.vcard = vobject.vCard()
-        c.vcard.add('n')
-        c.vcard.n.value = vobject.vcard.Name(given="%s"%gdc.title.text)
-        
-        c.vcard.add('fn')
-        c.vcard.fn.value = "%s"%gdc.title.text
-        #isinstance(gdc, atom.Entry)
-        ee_names = map(operator.attrgetter('tag'),gdc.extension_elements)
-        if len(gdc.extension_elements) >0:
-            for e in [e for e in ee_names if e == 'email']:
-                c.vcard.add('email')
-                c.vcard.email.value = gdc.extension_elements[ee_names.index('email')].attributes['address']
-                c.vcard.email.type_param = 'INTERNET'
-            for e in [e for e in ee_names if e == 'phoneNumber']:
-                c.vcard.add('tel')
-                c.vcard.tel.value = gdc.extension_elements[ee_names.index('phoneNumber')].text
-                c.vcard.tel.type_param = gdc.extension_elements[ee_names.index('phoneNumber')].attributes['rel'].split('#')[1]
-            for e in [e for e in ee_names if e == 'postalAddress']:
-                c.vcard.add('adr')
-                c.vcard.adr.value = vobject.vcard.Address(gdc.extension_elements[ee_names.index('postalAddress')].text)
-               # c.vcard.adr.value =
-                c.vcard.adr.type_param = gdc.extension_elements[ee_names.index('postalAddress')].attributes['rel'].split('#')[1]
-        
-        #c.vcard.add('uid').value = gdc.id.text
+        #get the gdata contact from google
+        try:
+            gc = self.service.Get(LUID, converter=gdata.contacts.ContactEntryFromString)
+        except gdata.service.RequestError:
+            return None
+            
+        c = self._conduit_contact_from_google_contact(gc)
         c.set_UID(LUID)
-        c.set_mtime(convert_madness_to_datetime(gdc.updated.text))
-        c.set_open_URI(gdc.link[1].href)
-        return c    
+        c.set_mtime(convert_madness_to_datetime(gc.updated.text))
+        return c
+
+    def _get_all_contacts(self):
+        feed = self.service.GetContactsFeed()
+        if not feed.entry:
+            return []
+        return [str(contact.id.text) for contact in feed.entry]
+        
+    def refresh(self):
+        DataProvider.TwoWay.refresh(self)
+        self._login()
+        if not self.loggedIn:
+            raise Exceptions.RefreshError("Could not log in")
 
     def get_all(self):
-        DataProvider.DataSource.get_all(self)
-        # we can ask the server for everything thats changed after a certain date, including deletions
+        DataProvider.TwoWay.get_all(self)
         self._login()
-        contacts = self.service.GetFeed(self.feed).entry
-        return [str(contact.id.text) for contact in contacts]
+        return self._get_all_contacts()
 
     def get(self, LUID):
-        DataProvider.DataSource.get(self, LUID)
-        return self._get_contact(LUID)
+        DataProvider.TwoWay.get(self, LUID)
+        self._login()
+        c = self._get_contact(LUID)
+        if c == None:
+            log.warn("Error getting/parsing gdata contact")
+        return c
+        
+    def put(self, data, overwrite, LUID=None):
+        #http://www.conduit-project.org/wiki/WritingADataProvider/GeneralPutInstructions
+        DataProvider.TwoWay.put(self, data, overwrite, LUID)
+        if overwrite and LUID:
+            LUID = self._update_contact(LUID, data)
+        else:
+            oldData = self._get_contact(LUID)
+            if LUID and oldData:
+                comp = data.compare(oldData)
+                #Possibility 1: If LUID != None (i.e this is a modification/update of a 
+                #previous sync, and we are newer, the go ahead an put the data
+                if LUID != None and comp == conduit.datatypes.COMPARISON_NEWER:
+                    LUID = self._update_contact(LUID, data)
+                #Possibility 3: We are the same, so return either rid
+                elif comp == conduit.datatypes.COMPARISON_EQUAL:
+                    return oldData.get_rid()
+                #Possibility 2, 4: All that remains are conflicts
+                else:
+                    raise Exceptions.SynchronizeConflictError(comp, data, oldData)
+            else:
+                #Possibility 5:
+                LUID = self._create_contact(data)
+                
+        #now return the rid
+        if not LUID:
+            raise Exceptions.SyncronizeError("Google contacts upload error.")
+        else:
+            return self._get_contact(LUID).get_rid()
+
 
     def delete(self, LUID):
+        DataProvider.TwoWay.delete(self, LUID)
         self._login()
-        self.service.Delete(LUID)
+        #get the gdata contact from google
+        try:
+            gc = self.service.Get(LUID, converter=gdata.contacts.ContactEntryFromString)
+            self.service.DeleteContact(gc.GetEditLink().href)
+        except gdata.service.RequestError, e:
+            log.warn("Error deleting: %s" % e)        
 
     def finish(self, aborted, error, conflict):
-        DataProvider.DataSource.finish(self)
+        DataProvider.TwoWay.finish(self)
 
     def configure(self, window):
         """
@@ -867,7 +987,249 @@ class ContactsSource(GoogleBase, DataProvider.DataSource):
             self._set_username(username.get_text())
             self._set_password(password.get_text())
         dlg.destroy()    
+        
+class _GoogleDocument:
+    def __init__(self, doc):
+        self.id = doc.id.text
+        #raw text version link
+        self.raw = doc.content.src
+        #edit link
+        self.link = doc.GetAlternateLink().href
+        self.title = doc.title.text.encode('UTF-8')
+        self.authorName = doc.author[0].name.text
+        self.authorEmail = doc.author[0].email.text
+        self.type = doc.category[0].label
+        
+        self.updated = convert_madness_to_datetime(doc.updated.text)
+        self.docid = self.get_document_id(self.link)
+        
+    # Parses the document id out of the alternate link url, the atom feed
+    # doesn't actually provide the document id
+    @staticmethod
+    def get_document_id(LUID):
+        parsed_url = urlparse.urlparse(LUID)
+        url_params = parsed_url[4]
+        document_id = url_params.split('=')[1]
+        return document_id
+        
+    def __str__(self):
+        return "%s:%s by %s (modified:%s) (id:%s)" % (self.type,self.title,self.authorName,self.updated,self.docid)
 
+class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
+    """
+    Contacts GData provider
+    
+    See: http://code.google.com/p/gdatacopier/source/browse/trunk/python/gdatacopier.py
+    """
+    _name_ = _("Google Documents")
+    _description_ = _("Sync your Google Documents")
+    _category_ = conduit.dataproviders.CATEGORY_OFFICE
+    _module_type_ = "twoway"
+    _out_type_ = "contact"
+    _icon_ = "applications-office"
+    
+    SUPPORTED_DOCUMENTS = ('DOC','ODT','SWX','TXT','RTF','HTM','HTML')
+    SUPPORTED_SPREADSHEETS = ('ODS','XLS','CSV','TSV')
+    SUPPORTED_PRESENTATIONS = ('PPT','PPS')
+    
+    TYPE_DOCUMENT = 'document'
+    TYPE_SPREADSHEET = 'spreadsheet'
+    TYPE_PRESENTATION = 'presentation'
+
+    def __init__(self, *args):
+        _GoogleBase.__init__(self,gdata.docs.service.DocsService())
+        DataProvider.DataSink.__init__(self)
+
+        self.documentFormat = 'ODT'
+        self.spreadsheetFormat = 'ODS'
+        self.presentationFormat = 'PPT'
+        
+        self._docs = {}
+        
+    def _upload_document(self, f):
+        name,ext = f.get_filename_and_extension()
+        ext = ext[1:].upper()
+
+        ms = gdata.MediaSource(
+                    file_path=f.get_local_uri(),
+                    content_type=gdata.docs.service.SUPPORTED_FILETYPES[ext])
+
+        #upload using the appropriate service
+        if ext in self.SUPPORTED_DOCUMENTS:
+            entry = self.service.UploadDocument(ms,name)
+        elif ext in self.SUPPORTED_SPREADSHEETS:
+            entry = self.service.UploadSpreadsheet(ms,name)
+        elif ext in self.SUPPORTED_PRESENTATIONS:
+            entry = self.service.UploadPresentation(ms,name)
+        else:
+            log.info("Unknown document format")
+            return None
+
+        return entry.id.text
+        
+    def _get_all_documents(self):
+        docs = {}
+        feed = self.service.GetDocumentListFeed()
+        if feed.entry:
+            for xmldoc in feed.entry:
+                docs[xmldoc.id.text] = _GoogleDocument(xmldoc)
+        return docs
+        
+    def _get_document(self, LUID):
+        if not LUID:
+            return None
+
+        #try cached doc first
+        if LUID in self._docs:
+            return self._docs[LUID]
+
+        #get the gdata contact from google
+        try:
+            xmldoc = self.service.GetDocumentListEntry(LUID)
+        except gdata.service.RequestError:
+            return None
+
+        return _GoogleDocument(xmldoc)
+        
+    def _download_doc(self, googleDoc):
+        docid = googleDoc.docid
+    
+        #print self.service.server
+        #return 
+        self.service.debug = True
+        
+        if googleDoc.type in ("document","presentation"):
+            format = "pdf"
+            #https://docs.google.com/MiscCommands?command=saveasdoc&exportformat=%s&docID=%s
+            resp = atom.service.HttpRequest(
+                                    service=self.service,
+                                    operation='GET',
+                                    data=None,
+                                    uri='/MiscCommands',
+                                    extra_headers={'Authorization':self.service._GetAuthToken()},
+                                    url_params={'command':'saveasdoc','exportformat':format,'docID':docid},
+                                    escape_params=True,
+                                    content_type='application/atom+xml')
+        elif False:#NOT WORKING googleDoc.type == "spreadsheet":
+            format = "xls"
+            #https://spreadsheets.google.com/ccc?output=%s&key=%s
+            #http://spreadsheets.google.com/fm?key=%s&fmcmd=4&hl=en
+            #self.service.server = "spreadsheets.google.com"
+            resp = atom.service.HttpRequest(
+                                    service=self.service,
+                                    operation='GET',
+                                    data=None,
+                                    uri='/ccc',
+                                    extra_headers={'Authorization':self.service._GetAuthToken()},
+                                    url_params={'output':format,'key':docid},
+                                    escape_params=True,
+                                    content_type='application/atom+xml')
+        else:
+            log.warn("Unknown format")
+            return None
+
+        path = "/home/john/Desktop/%s.%s" % (docid, format)
+        file_handle = open(path, 'wb')
+        file_handle.write(resp.read())
+        file_handle.close()
+        
+        return path
+        
+    def refresh(self):
+        DataProvider.DataSink.refresh(self)
+        self._login()
+        if not self.loggedIn:
+            raise Exceptions.RefreshError("Could not log in")
+            
+    def get_all(self):
+        self._docs = self._get_all_documents()
+        return self._docs.keys()
+        
+    def get(self, LUID):
+        pass
+
+#    def put(self, doc, overwrite, LUID=None):            
+#        #Check if we have already uploaded the document
+#        if LUID != None:
+#            info = self._get_photo_info(LUID)
+#            #check if a photo exists at that UID
+#            if info != None:
+#                if overwrite == True:
+#                    #replace the photo
+#                    return self._replace_photo(LUID, uploadInfo)
+#                else:
+#                    #Only upload the photo if it is newer than the Remote one
+#                    url = self._get_raw_photo_url(info)
+#                    remoteFile = File.File(url)
+#
+#                    #this is a limited test for equality type comparison
+#                    comp = photo.compare(remoteFile,True)
+#                    log.debug("Compared %s with %s to check if they are the same (size). Result = %s" % 
+#                            (photo.get_filename(),remoteFile.get_filename(),comp))
+#                    if comp != conduit.datatypes.COMPARISON_EQUAL:
+#                        raise Exceptions.SynchronizeConflictError(comp, photo, remoteFile)
+#                    else:
+#                        return conduit.datatypes.Rid(uid=LUID)
+#
+#        log.debug("Uploading Photo URI = %s, Mimetype = %s, Original Name = %s" % (photoURI, mimeType, originalName))
+#
+#        #upload the file
+#        return self._upload_photo (uploadInfo)
+
+    def delete(self, LUID):
+        DataProvider.DataSink.delete(self, LUID)
+        self._login()
+        #get the gdata contact from google
+        #try:
+        #    gc = self.service.Get(LUID, converter=gdata.contacts.ContactEntryFromString)
+        #    self.service.DeleteContact(gc.GetEditLink().href)
+        #except gdata.service.RequestError, e:
+        #    log.warn("Error deleting: %s" % e)        
+
+    def configure(self, window):
+        """
+        Configures the PicasaTwoWay
+        """
+        import gtk
+
+        def make_combo(widget, docType, val, values):
+            cb = widget.get_widget("%sCombo" % docType)
+            store = gtk.ListStore(str)
+            cell = gtk.CellRendererText()
+            
+            cb.set_model(store)
+            cb.pack_start(cell, True)
+            cb.add_attribute(cell, 'text', 0)
+            
+            for name in values:
+                rowref = store.append( (name,) )
+                if name == val:
+                    cb.set_active_iter(rowref)
+            
+
+        widget = Utils.dataprovider_glade_get_widget(
+                        __file__, 
+                        "documents-config.glade", 
+                        "GoogleDocumentsConfigDialog")
+                        
+        #get a whole bunch of widgets
+        username = widget.get_widget("username")
+        password = widget.get_widget("password")
+        
+        #preload the widgets        
+        username.set_text(self.username)
+        password.set_text(self.password)
+
+        #preload the combos
+        for i in (("document", self.documentFormat,self.SUPPORTED_DOCUMENTS),("spreadsheet", self.spreadsheetFormat,self.SUPPORTED_SPREADSHEETS),("presentation",self.presentationFormat,self.SUPPORTED_PRESENTATIONS)):
+            make_combo(widget, *i)
+        
+        dlg = widget.get_widget("GoogleDocumentsConfigDialog")
+        response = Utils.run_dialog (dlg, window)
+        if response == True:
+            self._set_username(username.get_text())
+            self._set_password(password.get_text())
+        dlg.destroy()   
 
 class YouTubeSource(DataProvider.DataSource):
     """
@@ -885,6 +1247,11 @@ class YouTubeSource(DataProvider.DataSource):
     USERS_FEED = "http://gdata.youtube.com/feeds/users"
     STD_FEEDS = "http://gdata.youtube.com/feeds/standardfeeds"
     VIDEO_NAME_RE = re.compile(r', "t": "([^"]+)"')
+    
+    #From: http://code.google.com/apis/youtube/dashboard/
+    UPLOAD_CLIENT_ID="ytapi-ConduitProject-Conduit-e14hdhdm-0"
+    UPLOAD_DEVELOPER_KEY="AI39si6wJ3VA_UWZCWeuA-wmJEpEhGbE3ZxCOZq89JJFy5CpSkFOq8gdZluNvBAM6DW8m7AhliSYPLyfEPJx6XphBq3vOBHuzQ"
+    UPLOAD_URL="http://uploads.gdata.youtube.com/feeds/api/users/%(username)s/uploads"
 
     def __init__(self, *args):
         DataProvider.DataSource.__init__(self)
